import json
import re
import traceback
import tornado.web
import tornado.escape
from tornado.options import options
import uuid
import hashlib
import base64
# from models.user import User


class MyEncoder(json.JSONEncoder):
    """ Custom JSON encoder with auto decode b type data in dict"""
    def default(self, obj):
        """

        :param obj: 

        """
        if isinstance(obj, (bytes, bytearray)):
            return obj.decode("utf-8")
        elif type(obj).__module__ == "numpy":
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)


class Controller(tornado.web.RequestHandler):
    """ Base Controller Class"""

    def get_current_user(self):
        """ """
        pass
        # try:
        #     secret = self.get_secure_cookie('secret')
        #     user = self.decrypt_string(secret, self.user_model.secret)

        #     if self.user_model.user_exist(user):
        #         return self.user_model.users[user]
        #     else:
        #         return None
        # except:
        #     return None

    def prepare(self):
        """ Prepare args of request to dict"""
        try:
            if self.request.headers.get("Content-Type") and self.request.headers.get("Content-Type").startswith("application/json"):
                self.args = json.loads(self.request.body)
            else:
                arguments = (
                    self.request.arguments or self.request.query_arguments)
                self.args = {k: arguments[k][0].decode() for k in arguments}

            # self.args.pop('_xsrf')
        except:
            self.args = None

    def args_required(self, args_list):
        """
        Check request arguments (all arguments in args_list should be in request arguments)
        :param args_list: 

        """
        if self.args and len(self.args.keys()) > 0:
            return all(x in self.args.keys() for x in args_list)

    def args_required_strict(self, args_list):
        """
        Check request arguments in strict mode (only arguments in args_list should be in request arguments)
        :param args_list: 

        """
        if self.args and len(self.args.keys()) > 0:
            return all(x in self.args.keys() for x in args_list) and len(self.args.keys()) == len(args_list)

    def error_message(self, etype, val=""):
        """
        Create HTTP response with error etype and val
        :param etype: 
        :param val:  (Default value = "")

        """
        self.clear()
        self.set_status(etype)
        error = {
            'message': val
        }
        self.write(tornado.escape.json_encode(error))

    def write_json(self, data):
        """
        Create HTTP response with json data
        :param data: 

        """
        self.set_header("Content-Type", 'application/json')
        self.write(json.dumps({"data": data or []}, cls=MyEncoder))


    def crypt_rc4(self, data, key):
        """
        Simple RC4 encryption 
        :param data: 
        :param key: 
        :returns: crypted data
        :rtype: bytes
        """
        c = 0
        box = list(range(256))
        for i in list(range(256)):
            c = (c + box[i] + ord(key[i % len(key)])) % 256
            box[i], box[c] = box[c], box[i]
        c = 0
        y = 0
        out = []
        for char in data:
            c = (c + 1) % 256
            y = (y + box[c]) % 256
            box[c], box[y] = box[y], box[c]
            out.append(chr(ord(char) ^ box[(box[c] + box[y]) % 256]))

        return ''.join(out)

    def crypt_string(self, data, key):
        """
        RC4 encryption to base64 string
        :param data: 
        :param key: 
        :returns: crypted data
        :rtype: string
        """
        data = self.crypt_rc4(data, key)
        data = base64.b64encode(bytes(data, 'utf-8'))
        data = str(data, 'utf-8')
        return data

    def decrypt_string(self, data, key):
        """
        RC4 decrypt base64 string
        :param data: 
        :param key: 
        :returns: decrypted data
        :rtype: string
        """
        data = base64.b64decode(data)
        data = self.crypt_rc4(str(data, 'utf-8'), key)
        return data

    def crypt_sha256(self, data):
        """

        :param data: 

        """
        data = bytes(data, 'utf-8')
        hash_object = hashlib.sha256(data)
        hex_dig = hash_object.hexdigest()
        return hex_dig

    def generate_uuid(self):
        """ Generate uuid """
        return str(uuid.uuid4())
