import hashlib
from requests.api import get
import tornado
from tornado.escape import json_encode
import requests, json
from random import randrange

from base import controller
from models.dbQuery import db_query

import trimesh
import numpy as np

from settings import STORAGE_URL

UPLOAD_PATH = STORAGE_URL + 'upload'

MANIFEST_TEMP = {
    "_id": 0,
    "manifest": {
        "properties": {
            "children": [
                {
                    "object_type": 3,
                    "props": {
                        "transformation": [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1],
                    },
                },
            ],
            "transformation": [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1],
            "hp": 100,
        },
        "rules": [
            {
                "conditions": [
                    {
                        "prop_name": "is_ghost",
                        "comparator": "eq",
                        "value": False,
                    },
                ],
                "effect": {
                    "view_id": 0,
                },
                "priority": 0,
            },
            {
                "conditions": [
                    {
                        "prop_name": "is_ghost",
                        "comparator": "eq",
                        "value": True,
                    },
                ],
                "effect": {
                    "view_id": 1,
                },
                "priority": 0,
            },
        ],
        "meta": {
            "id": "",
            "name": "new",
            "tags": "",
            "size": "",
            "description": "",
            "views": {
            },
            "is_prefab": False,
            "is_instanced": False,
            "permissions": {
            },
            "preview": "",
            "effects": {}
        }
    }
}

class Block(controller.Controller):
    # @tornado.web.authenticated
    def get(self, bclass):
        is_new = False
        data = {}
        if bclass == "new":
            data = MANIFEST_TEMP
            data["_id"] = randrange(0, 1<<63)
            data["manifest"]["meta"]["id"] = str(data["_id"])
            is_new = True
        else:
            data = db_query.get_block(bclass)

        self.render("block.html", data=data, json=json, is_new = is_new)

    def post(self, bclass):
        data = {}

        preview_path = None

        # save assets
        url = UPLOAD_PATH
        files = self.request.files

        for dst in files:
            params = {'path': self.args["_id"] +
                    "/assets/" + dst, 'nohash': 1, 'force':1}

            if dst == "preview":
                params = {'path': "avvy_assets"}

            for f in files[dst]:
                fdata = {"file": (f["filename"], f["body"], f["content_type"])}
                r = requests.post(UPLOAD_PATH, data=params, files=fdata)

                filename = r.json()["files"]["file"]
                if dst == "preview":
                    preview_path = STORAGE_URL + "storage/" + filename

        properties_data = json.loads(self.args.pop("props"))
        properties = properties_data.pop("properties")
        rules = properties_data.pop("rules")
        meta = properties_data.pop("meta")


        data["_id"] = self.args["_id"]
        data["manifest"] = {}
        data["manifest"]["properties"] = properties
        data["manifest"]["rules"] = rules
        data["manifest"]["meta"] = meta

        data["manifest"]["meta"]["id"] = self.args["_id"]
        data["manifest"]["meta"]["name"] = self.args["name"]
        data["manifest"]["meta"]["description"] = self.args["description"]
        data["manifest"]["meta"]["size"] = self.args["size"]
        data["manifest"]["meta"]["tags"] = self.args["tags"]
        data["manifest"]["meta"]["is_prefab"] = True if self.args.get("is_prefab", False) else False
        data["manifest"]["meta"]["is_instanced"] = True if self.args.get("is_instanced", False) else False
        if data["manifest"]["meta"].get("permissions", False):
            data["manifest"]["meta"]["permissions"]["is_allow_fill"] = True if self.args.get("is_allow_fill", False) else False
        else: 
            data["manifest"]["meta"]["permissions"] = {}
            data["manifest"]["meta"]["permissions"]["is_allow_fill"] = True if self.args.get("is_allow_fill", False) else False
        
        if preview_path:
            data["manifest"]["meta"]["preview"] = preview_path

        if data["manifest"]["meta"].get("effects", False):
            for effects in data["manifest"]["meta"]["effects"]:
                effectsId = data["manifest"]["meta"]["effects"][effects]
                for effect in effectsId:
                    str_hashed = effect["name"] + str(effect["args"])
                    
                    hash_object = hashlib.sha256(str_hashed.encode('utf-8'))
                    hex_dig = hash_object.hexdigest()
                    effect["hash"] = hex_dig

        
        db_query.add_block(data)

        r = requests.post(
            url,
            data={'path': self.args["_id"], 'nohash': 1, 'force':1},
            files={
                "file": (self.args["_id"] + ".manifest", json.dumps(data["manifest"]))
            })
        
        self.redirect("/block/" + self.args["_id"])

    # @tornado.web.authenticated

    def delete(self, bclass):
        db_query.delete_block(bclass)
        self.write_json({"status": True})
