import tornado
from tornado.escape import json_encode
import requests, json
from random import randrange

from base import controller
from models.dbQuery import db_query

import trimesh
import numpy as np

from settings import STORAGE_URL

UPLOAD_PATH = STORAGE_URL + 'upload'

def as_mesh(scene_or_mesh):
    if isinstance(scene_or_mesh, trimesh.Scene):
        mesh = trimesh.util.concatenate([
            trimesh.Trimesh(vertices=m.vertices, faces=m.faces)
            for m in scene_or_mesh.geometry.values()])
    else:
        mesh = scene_or_mesh
    return mesh

def compute_cels(path):
    scene = trimesh.exchange.load.load_remote(path)
    mesh = as_mesh(scene)
    # geometries = list(mesh.geometry.values())

    voxelized_mesh = mesh.voxelized(pitch=0.100001).fill(method='holes')

    transform = np.array([1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1])

    cells = []
    for point in voxelized_mesh.points:
        cell = {}
        cell["object_type"] = 3
        cell["props"] = {}
        cell_transform = transform.copy()
        cell_transform[12] = round(point[0] * 10)
        cell_transform[13] = round(point[1] * 10)
        cell_transform[14] = round(point[2] * 10)
        cell["props"]["transformation"] = cell_transform.tolist()
        cells.append(cell)
    return cells

class View(controller.Controller):
    # @tornado.web.authenticated

    def post(self, bclass):

        data = db_query.get_block(bclass)

        children = []
        obj_path = ''

        # save assets
        file = self.request.files["asset"][0]

        files_data = {"file": (file["filename"], file["body"], file["content_type"])}
        r = requests.post(UPLOAD_PATH, data={'path': "avvy_assets"}, files=files_data)

        filename = r.json()["files"]["file"]
        obj_path = STORAGE_URL + "storage/" + filename

        print(obj_path)

        if int(self.args["id"]) == 0:
            children = compute_cels(obj_path)
            data["manifest"]["properties"]["children"] = children

        if not data["manifest"]["meta"].get("views", False):
            data["manifest"]["meta"]["views"] = {}

        data["manifest"]["meta"]["views"][self.args["id"]] = obj_path

        db_query.add_block(data)

        r = requests.post(
            UPLOAD_PATH,
            data={'path': bclass, 'nohash': 1, 'force':1},
            files={
                "file": (bclass + ".manifest", json.dumps(data["manifest"]))
            })


        self.redirect("/block/" + bclass)

    # @tornado.web.authenticated

    def delete(self, bclass, view_id):
        data = db_query.get_block(bclass)
        del data["manifest"]["meta"]["views"][view_id]
        db_query.add_block(data)
        self.write_json({"status": True, "redirect": "/block/" + bclass})
