(function ($) {
  "use strict";

  const STORAGE_PATH = "https://storage.avvyland.com/storage";

  const delCellBtn = document.getElementById("delete-cell");
  const delViewBtn = document.getElementById("remove-view");
  const delBlockBtn = document.getElementById("button-delete");
  const createBlockBtn = document.getElementById("create-block");
  const submitBtn = document.getElementById("submit-button");
  const approveForm = document.getElementById("approve-form");
  const inputBlockProps = document.getElementById("inputBlockProps");

  const viewsContainer = document.getElementById("views");
  const effectsContainer = document.getElementById("effects");
  const newEffectBtn = document.getElementById("new-effect");
  const addNewEffectBtn = document.getElementById("add-new-effect");
  const effectEditor = document.getElementById("jsoneditor-effect");
  const manifestEditor = document.getElementById("jsoneditor");

  const btnState = {
      active: false
  };
  let newBlockProps = {};

  let scene;
  // let mesh;
  let activeView = "0";
  if (newEffectBtn) {
    newEffectBtn.textContent = `New Effect For ${activeView} View ID`;
  }
  

  const viewMeshes = {};
  const effects = {};
  let effect;
  let effectJSONEditor;

  function renderEffectEditor() {
    newEffectBtn.classList.add("d-none");
    effectEditor.classList.remove("d-none");
    addNewEffectBtn.classList.remove("d-none");

    if (!effectJSONEditor) {
      effectJSONEditor = new JSONEditor(effectEditor, {
        modes: ["tree", "code"],
        onChange: function (data) {
          effect = JSON.parse(effectJSONEditor.getText());
        },
      });
    }
    const json = {
      name: "",
      type: "",
      args: {}
    };
    effectJSONEditor.set(json);

    // $("#inputBlockProps").val(effectJSONEditor.getText());
  }

  function addNewEffect() {
    newEffectBtn.classList.remove("d-none");
    effectEditor.classList.add("d-none");
    addNewEffectBtn.classList.add("d-none");

    const props = JSON.parse(inputBlockProps.value);
    if (props.meta.effects && props.meta.effects[activeView] && props.meta.effects[activeView].length) {
      props.meta.effects[activeView].push(effect);
    } else {
      if (!props.meta.effects) {
        props.meta.effects = {};
      }
      props.meta.effects[activeView] = [];
      props.meta.effects[activeView].push(effect);
    }

    inputBlockProps.value = JSON.stringify(props);
    window.AvvY.blockProps = props;

    approveForm.submit();
  }

  function loadView(objUrl) {
    if (objUrl) {
        loader.load(
        objUrl,
        function (gltf) {
            viewMeshes[activeView] = findAll(gltf.scene.children);
            viewMeshes[activeView].rotation.set(0, 0, 0);
            viewMeshes[activeView].scale.set(100, 100, 100);
            viewMeshes[activeView].ignoreRaycaster = true;
            scene.add(viewMeshes[activeView]);

        },
        function (xhr) {
            console.log((xhr.loaded / xhr.total) * 100 + "% loaded");
        },
        function (error) {
            console.log("An error happened");
        }
        );
    }
}

  function changeActiveView(view) {
    const views = document.querySelectorAll(".card-view__wrapper");
    for (let i = 0; i < views.length; i++) {
        views[i].classList.remove("active");
    }
    scene.remove(viewMeshes[activeView]);
    activeView = view;
    newEffectBtn.textContent = `New Effect For ${activeView} View ID`;
    if (viewMeshes[activeView]) {
        scene.add(viewMeshes[activeView]);
    } else {
        loadView(AvvY.blockProps["meta"]["views"][activeView]);
    }
}

  function renderViewsId(views) {
    if (viewsContainer) {
      const template = document.getElementById("temp__view");
      viewsContainer.innerHTML = "";

      for (const viewId in views) {
          const viewWrapper = template.content.getElementById("view__wrapper").cloneNode(true);
          if (activeView === viewId) {
              viewWrapper.classList.add("active");
          }
          viewWrapper.addEventListener("click", e => {
              changeActiveView(viewId);
              viewWrapper.classList.add("active");
          });
          
          const viewIdEl = viewWrapper.querySelector("#view-id");
          const viewUrlEl = viewWrapper.querySelector("#view-url");

          viewIdEl.textContent = viewId;
          viewUrlEl.textContent = views[viewId];

          viewsContainer.append(viewWrapper);
      }
    }
  }

  function renderDelEffectBtn(el, id, hash) {
    let btn = document.getElementById("delete-effect-button");
    if (btn) {
      btn.remove();
    }
    btn = document.createElement("button");
    btn.id = "delete-effect-button";
    btn.classList.add("btn", "btn-danger");
    btn.textContent = "Delete this effect";

    btn.addEventListener("click", () => {
      const newEffects = window.AvvY.blockProps.meta.effects[id].filter((effect) => effect.hash !== hash);
      window.AvvY.blockProps.meta.effects[id] = newEffects;

      inputBlockProps.value = JSON.stringify(AvvY.blockProps);
      approveForm.submit();
    });

    el.append(btn);
  }

  function renderEffects(effects) {
    if (effectsContainer) {
      const template = document.getElementById("temp__effects");
      const propsTemplate = document.getElementById("temp__props-effects");
      effectsContainer.innerHTML = "";

      for (const effectsId in effects) {
        if (effects[effectsId].length > 0) {
          const viewWrapper = template.content.getElementById("view__wrapper").cloneNode(true);
          const viewIdEl = viewWrapper.querySelector(".effect-view-id");
          const effectsProps = viewWrapper.querySelector(".card-view__props");
          viewIdEl.textContent = effectsId;
        
          effectsContainer.append(viewWrapper);

          for (let i = 0; i < effects[effectsId].length; i++) {
            const effect = effects[effectsId][i];
            const propsWrapper = propsTemplate.content.querySelector(".props-effects").cloneNode(true);
            const effectName = propsWrapper.querySelector(".effect-name");
            const effectType = propsWrapper.querySelector(".effect-type");
            const effectArgs = propsWrapper.querySelector(".effect-args");

            effectName.textContent = effect.name;
            effectType.textContent = effect.type;
            effectArgs.textContent = JSON.stringify(effect.args);

            propsWrapper.addEventListener("click", (e) => {
              renderDelEffectBtn(propsWrapper, effectsId, effect.hash);
            });

            effectsProps.append(propsWrapper);
          }
        }
      }
    }
  }
  
  renderViewsId(window.AvvY.blockProps.meta.views);
  renderEffects(window.AvvY.blockProps.meta.effects);

  if (manifestEditor) {
    const editor = new JSONEditor(manifestEditor, {
      modes: ["tree", "code"],
      onChange: function (data) {
        // inputBlockProps.value = JSON.stringify(editor.getText());
        inputBlockProps.value = editor.getText();
        window.AvvY.blockProps = JSON.parse(editor.getText());
      },
    });
    // editor._debouncedValidate = function(){return true;}
    const json = {
      properties: {
        children: [
          {
            object_type: 3,
            props: {
              transformation: [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1],
            },
          },
        ],
        transformation: [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 1],
        hp: 100,
      },
      rules: [
        {
          conditions: [
            {
              prop_name: "is_ghost",
              comparator: "eq",
              value: false,
            },
          ],
          effect: {
            view_id: 0,
          },
          priority: 0,
        },
        {
          conditions: [
            {
              prop_name: "is_ghost",
              comparator: "eq",
              value: true,
            },
          ],
          effect: {
            view_id: -1,
          },
          priority: 0,
        },
      ],
    };
    editor.set(
      Object.keys(window.AvvY.blockProps).length ? window.AvvY.blockProps : json
    );
    // $("#inputBlockProps").val(editor.getText());
    inputBlockProps.value = editor.getText();
  }

  const changeStateBtn = function(e) {
    btnState.active = !btnState.active;
    if (btnState.active) {
        delCellBtn.classList.add("active-button");
        if (viewMeshes[activeView].children.length) {
          for (let i = 0; i < viewMeshes[activeView].children.length; i++) {
            const child = viewMeshes[activeView].children[i];
            child.material.depthTest = false;
          }
        } else {
          viewMeshes[activeView].material.depthTest = false;
        }
        
    } else {
        delCellBtn.classList.remove("active-button");
        if (viewMeshes[activeView].children.length) {
          for (let i = 0; i < viewMeshes[activeView].children.length; i++) {
            const child = viewMeshes[activeView].children[i];
            child.material.depthTest = true;
          }
        } else {
          viewMeshes[activeView].material.depthTest = true;
        }
    }
  };

  const raycast = function() {

    var raycaster = new THREE.Raycaster();
    var mouse = new THREE.Vector2();

    let intersects, mesh, result, camera, renderer, context;


    return function(event) {
        const points = [];

        context = this;

        camera = engine.camera;
        renderer = engine.renderer;

        // mouse.x = (event.x / renderer.domElement.clientWidth) * 2 - 1;
        // mouse.y = -(event.y / renderer.domElement.clientHeight) * 2 + 1;
        
        raycaster.setFromCamera(event, camera);
        intersects = raycaster.intersectObject(engine.view, true);

        for (var i = 0; i < intersects.length; i++) {
            mesh = intersects[i].object;
            if (mesh.ignoreRaycaster || mesh.parent.ignoreRaycaster)
                continue;
            else {
                result = intersects[i];
                result.object = mesh;
                return result;
            }
        }
    }
  }();

  delCellBtn.addEventListener("click", changeStateBtn);
  if (newEffectBtn) {
    newEffectBtn.addEventListener("click", renderEffectEditor);
  }
  if (addNewEffectBtn) {
    addNewEffectBtn.addEventListener("click", addNewEffect);
  }
  if (delViewBtn) {
    delViewBtn.addEventListener("click", (e) => {
      // delViewBtn.disabled = true;
      window.AvvY.createRequest({
        url: `${window.AvvY.deleteLink}/view/${activeView}`,
        method: "DELETE",
        onSuccess: function (result) {
          document.location.href = result.data.redirect;
        },
      });
    });
  }
  if (delBlockBtn) {
    delBlockBtn.addEventListener("click", (e) => {
      // delBlockBtn.disabled = true;
      window.AvvY.createRequest({
        url: window.AvvY.deleteLink,
        method: "DELETE",
        onSuccess: function (result) {
          document.location.href = window.AvvY.redirectLink;
        },
      });
    });
  }
  if (createBlockBtn) {
    createBlockBtn.addEventListener("click", (e) => {
      approveForm.submit();
    });
  }


  var engine = new VG.Engine("threeview");

  var mouse = new VG.MouseEventsHandler(engine.renderer.domElement);

  var cameraController = new VG.CameraControllerOrbit(
    engine.camera,
    engine.renderer.domElement
  );
  cameraController.maxPolarAngle = Math.PI / 2;
  cameraController.minDistance = 1;
  cameraController.maxDistance = 100;

  var light = new THREE.DirectionalLight(0xffffffff, 0.7);
  light.position.set(500, 300, 100);
  engine.view.add(light);

  var alight = new THREE.AmbientLight(0xffffffff, 1.2);
  engine.view.add(alight);

  scene = new THREE.Object3D();
  scene.scale.set(0.1, 0.1, 0.1);
  engine.view.add(scene);

  const loader = new THREE.GLTFLoader();

  // Load a glTF resource

  var cellGeometry = new THREE.BoxGeometry(0.95, 0.95, 0.95);
  var cellMaterial = new THREE.MeshLambertMaterial({
    color: "green",
    transparent: true,
    opacity: 0.2,
  });
  var matrix = new THREE.Matrix4();

  for (let i = 0; i < AvvY.blockProps.properties.children.length; i++) {
    newBlockProps[i] = AvvY.blockProps.properties.children[i];
  }

  for (var ch in AvvY.blockProps.properties.children) {
    var cell = AvvY.blockProps.properties.children[ch];
    let mesh = new THREE.Mesh();
    mesh.geometry = cellGeometry;
    mesh.material = cellMaterial;

    mesh.userData.id = ch;
    mesh.userData.cell = cell;

    matrix.fromArray(cell.props.transformation);
    mesh.position.setFromMatrixPosition(matrix);
    mesh.rotation.setFromRotationMatrix(matrix);
    engine.view.add(mesh);
  }

  var objUrl = AvvY.blockProps?.meta?.views?.[0];

  function findAll(children) {
    const meshes = [];
    function _find(children) {
        for (let i = 0; i < children.length; i++) {
            let child = children[i];
            if (child.isMesh) {
                meshes.push(child);

            } else {
                child = _find(child.children);
                if (child) {
                    meshes.push(child);
                }
            }
        }
    }
    _find(children);
    
    const object = new THREE.Object3D();
    for (let i = 0; i < meshes.length; i++) {
        object.add(meshes[i]);
    }
    return object;
  }

  if (objUrl) {
    loadView(objUrl);
  }

  function deleteOneCell(e) {
    if (btnState.active) {
      let cell;
      cell = raycast(mouse.mousePosition);
      if (cell) {
        engine.view.remove(cell.object);
        delete newBlockProps[cell.object.userData.id];
      } else {
        return null;
      }
    }
}
  
function saveCells() {
    const _newBlockProps = [];
    for (const id in newBlockProps) {
        _newBlockProps.push(newBlockProps[id]);
    }
    return _newBlockProps;
}

VG.EventDispatcher.bind("mouse.click", {}, deleteOneCell);

submitBtn.addEventListener("click", e => {
  e.preventDefault();
  const newCells = saveCells();

  AvvY.blockProps["properties"]["children"] = newCells;
  inputBlockProps.value = JSON.stringify(AvvY.blockProps);
  approveForm.submit();
});


})(jQuery);