const dropContainer = document.getElementById("drop-container");
const dropInput = document.getElementById("drop-input");
const dropIcon = document.getElementById("drop-icon");
const dropItem = document.getElementById("drop-item");
const dropDesc = document.getElementById("drop-desc");

const blockNameInput = document.getElementById("block-name");
const blockTagsInput = document.getElementById("block-tags");
const blockDescInput = document.getElementById("block-desc");
const nextBtn = document.getElementById("next-button");
const submitBtn = document.getElementById("submit-button");

const viewsContainer = document.getElementById("views");
const errorSection = document.getElementById("error-section");

const delCellBtn = document.getElementById("delete-cell");
const btnState = {
    active: false
};


const previewBtn = document.getElementById("preview-button");
const newViewBtn = document.getElementById("new-view-button");
const inputNewBlockId = document.getElementById("inputNewBlockId");
const inputBlockAssetPreview = document.getElementById("inputBlockAssetPreview");
const newViewForm = document.getElementById("new-view-form");
const saveNewViewBtn = document.getElementById("save-new-view");
const inputBlockAsset = document.getElementById("inputBlockAsset");
const inputBlockName = document.getElementById("inputBlockName");
const previewImage = document.getElementById("preview-image");
const inputBlockDescription = document.getElementById("inputBlockDescription");
const inputBlockTags = document.getElementById("inputBlockTags");
const inputBlockProps = document.getElementById("inputBlockProps");
const inputBlockId = document.getElementById("inputBlockId");
const inputBlockSize = document.getElementById("inputBlockSize");
const inputIsPrefab = document.getElementById("inputIsPrefab");
const approveForm = document.getElementById("approve-form");
const addNewViewModal = document.getElementById("add-view-modal");

const fileNameField = document.getElementById("file-name");
const viewFileNameField = document.getElementById("file-name-view");
const nextView = document.getElementById("next-view");

const deleteViewBtn = document.getElementById("remove-view")

const dropWrapper = document.getElementById("drop-wrapper");
const formWrapper = document.getElementById("form-wrapper");

nextBtn.disabled = true;

let formData = new FormData();
const newBlockState = {
    file: false,
    name: false,
    tags: false,
    desc: false,
}
let newBlockProps = {};
let activeView = "0";

const viewMeshes = {};

let mouse;
let engine;
let scene;
const loader = new THREE.GLTFLoader();
validateState();

const raycast = function() {

    var raycaster = new THREE.Raycaster();
    var mouse = new THREE.Vector2();

    var intersects, mesh, result, camera, renderer, context;


    return function(event) {
        const points = [];

        context = this;

        camera = engine.camera;
        renderer = engine.renderer;

        // mouse.x = (event.x / renderer.domElement.clientWidth) * 2 - 1;
        // mouse.y = -(event.y / renderer.domElement.clientHeight) * 2 + 1;
        
        raycaster.setFromCamera(event, camera);
        intersects = raycaster.intersectObject(engine.view, true);

        for (var i = 0; i < intersects.length; i++) {
            mesh = intersects[i].object;
            if (mesh.ignoreRaycaster || mesh.parent.ignoreRaycaster)
                continue;
            else {
                result = intersects[i];
                result.object = mesh;
                return result;
            }
        }
    }
}();

function validateState() {
    if (newBlockState.file && newBlockState.name && newBlockState.tags && newBlockState.desc) {
        nextBtn.classList.remove("disabled");
        nextBtn.disabled = false;
    } else {
        nextBtn.classList.add("disabled");
        nextBtn.disabled = true;
    }
}

dropContainer.addEventListener("click", (e) => {
    dropInput.click();
});

dropContainer.addEventListener("dragenter", e => {
    e.preventDefault();
    e.stopPropagation();
    // add style
});

dropContainer.addEventListener("dragover", e => {
    e.preventDefault();
    e.stopPropagation();
    // add style
});

dropContainer.addEventListener("dragleave", e => {
    e.preventDefault();
    e.stopPropagation();
    // remove style
});

dropContainer.addEventListener("drop", e => {
    e.preventDefault();
    e.stopPropagation();
    // remove style

    let file = e.dataTransfer.files[0];
    addFile(file);
    
});

dropInput.addEventListener("change", e => {
    if (dropInput.files[0]) {
        addFile(dropInput.files[0]);
    }
});

function addFile(file) {
    dropIcon.style.display = "none";
    dropItem.style.display = "block";
    dropContainer.style.background = "#e8f0fe";
    dropDesc.textContent = file.name;
    newBlockState.file = true;
    validateState();
    formData.append('asset', file);
    
}

function checkState(el, key) {
    if (el.value) {
        newBlockState[key] = true; 
    } else {
        newBlockState[key] = false;
    }
    validateState();
}

blockNameInput.addEventListener("input", e => {
    checkState(blockNameInput, "name");
});
blockTagsInput.addEventListener("input", e => {
    checkState(blockTagsInput, "tags");
});
blockDescInput.addEventListener("input", e => {
    if (blockDescInput.value.length) {
        blockDescInput.style.background = "#e8f0fe";
    } else {
        blockDescInput.style.background = "#ffffff";
    }
    checkState(blockDescInput, "desc");
});

nextBtn.addEventListener("click", e => {
    e.preventDefault();

    formData.append("name", blockNameInput.value);
    formData.append("tags", blockTagsInput.value.trim().split(" ").toString());
    formData.append("desc", blockDescInput.value);

    const _xsrf = document.querySelector("input[name='_xsrf']").value;
    formData.append("_xsrf",_xsrf);
    fetch(`/user/block/new`, {
      method: 'POST',
      body: formData,
    })
    .then((res) => {
        return res.json();
    })
    .then(response => {
        dropWrapper.remove();
        formWrapper.remove();

        const data = response.data;
        AvvY.deleteLink = `/block/${data["manifest"]["meta"]["id"]}`;
        AvvY.redirectLink = "/";
        AvvY.blockProps = data["manifest"];
        AvvY.class_id = data["manifest"]["meta"]["id"];

        for (let i = 0; i < AvvY.blockProps.properties.children.length; i++) {
            newBlockProps[i] = AvvY.blockProps.properties.children[i];
        }

        inputBlockName.value = data["manifest"]["meta"]["name"];
        inputBlockDescription.value = data["manifest"]["meta"]["description"];
        inputBlockProps.value = JSON.stringify(data["manifest"]);
        inputBlockId.value =  data["manifest"]["meta"]["id"];
        inputBlockSize.value = data["manifest"]["meta"]["size"];
        inputIsPrefab.value = false;

        for (let i = 0; i < data["manifest"]["meta"]["tags"].length; i++) {
            const tag = data["manifest"]["meta"]["tags"][i];
            $("#inputBlockTags").tagsinput('add', tag);
        }

        nextView.style.display = "block";
        renderViewsId(data["manifest"]["meta"]["views"]);
        renderView();
    })
    .catch(() => {
        dropWrapper.remove();
        formWrapper.remove();

        errorSection.classList.remove("d-none");
    });
});

previewBtn.addEventListener("click", (e) => {
    e.preventDefault();
    inputBlockAssetPreview.click();
});
newViewBtn.addEventListener("click", (e) => {
    e.preventDefault();
    inputBlockAsset.click();
});
inputBlockAssetPreview.addEventListener("change", () => {
    fileNameField.textContent = inputBlockAssetPreview.files[0].name.length > 30 ? `...${inputBlockAssetPreview.files[0].name.slice(-30)}` : inputBlockAssetPreview.files[0].name;
    let reader = new FileReader()
    reader.readAsDataURL(inputBlockAssetPreview.files[0]);
    reader.onloadend = function() {
        previewImage.src = reader.result;
    }
});
inputBlockAsset.addEventListener("change", () => {
    viewFileNameField.textContent = inputBlockAsset.files[0].name.length > 30 ? `...${inputBlockAsset.files[0].name.slice(-30)}` : inputBlockAsset.files[0].name;
});

deleteViewBtn.addEventListener("click", (e) => {
    e.preventDefault();
    deleteViewBtn.disabled = true;

    let formData = new FormData();
    const _xsrf = document.querySelector("input[name='_xsrf']").value;
    formData.append("_xsrf",_xsrf);

    fetch( `/user${AvvY.deleteLink}/view/${activeView}`, {
        method: 'DELETE',
        body: formData,
      })
      .then((res) => {
          return res.json();
      })
      .then(response => {
        const data = response.data;
        scene.remove(viewMeshes[activeView]);
        delete viewMeshes[activeView];
        renderViewsId(data["manifest"]["meta"]["views"]);
        AvvY.blockProps["meta"]["views"] = data["manifest"]["meta"]["views"];
          
      })
      .catch(() => {
        nextView.style.display = "none";
        errorSection.classList.remove("d-none");
      })

});

function loadView(objUrl) {
    if (objUrl) {
        loader.load(
        objUrl,
        function (gltf) {
            viewMeshes[activeView] = findAll(gltf.scene.children);
            viewMeshes[activeView].rotation.set(0, 0, 0);
            viewMeshes[activeView].scale.set(100, 100, 100);
            viewMeshes[activeView].ignoreRaycaster = true;
            scene.add(viewMeshes[activeView]);

        },
        function (xhr) {
            console.log((xhr.loaded / xhr.total) * 100 + "% loaded");
        },
        function (error) {
            console.log("An error happened");
        }
        );
    }
}

function renderView () {
    engine = new VG.Engine("threeview");
    mouse = new VG.MouseEventsHandler(engine.renderer.domElement);

    const cameraController = new VG.CameraControllerOrbit(
        engine.camera,
        engine.renderer.domElement
    );
    cameraController.maxPolarAngle = Math.PI / 2;
    cameraController.minDistance = 1;
    cameraController.maxDistance = 100;

    const light = new THREE.DirectionalLight(0xffffffff, 0.7);
    light.position.set(500, 300, 100);
    engine.view.add(light);

    const alight = new THREE.AmbientLight(0xffffffff, 1.2);
    engine.view.add(alight);

    scene = new THREE.Object3D();
    scene.scale.set(0.1, 0.1, 0.1);
    engine.view.add(scene);

    // Load a glTF resource
    const cellGeometry = new THREE.BoxGeometry(0.95, 0.95, 0.95);
    const cellMaterial = new THREE.MeshLambertMaterial({
        color: "green",
        transparent: true,
        opacity: 0.2,
    });
    var matrix = new THREE.Matrix4();

    for (let ch in AvvY.blockProps.properties.children) {
        let cell = AvvY.blockProps.properties.children[ch];
        let mesh = new THREE.Mesh();
        mesh.geometry = cellGeometry;
        mesh.material = cellMaterial;

        mesh.userData.id = ch;
        mesh.userData.cell = cell;

        matrix.fromArray(cell.props.transformation);
        mesh.position.setFromMatrixPosition(matrix);
        mesh.rotation.setFromRotationMatrix(matrix);
        engine.view.add(mesh);
    }

    let objUrl = AvvY.blockProps?.meta?.views?.[0];
    loadView(objUrl);
    
}

function changeActiveView(view) {
    const views = document.querySelectorAll(".card-view__wrapper");
    for (let i = 0; i < views.length; i++) {
        views[i].classList.remove("active");
    }
    scene.remove(viewMeshes[activeView]);
    activeView = view;
    if (viewMeshes[activeView]) {
        scene.add(viewMeshes[activeView]);
    } else {
        loadView(AvvY.blockProps["meta"]["views"][activeView]);
    }
}

function renderViewsId(views) {
    const template = document.getElementById("temp__view");
    viewsContainer.innerHTML = "";

    for (const viewId in views) {
        const viewWrapper = template.content.getElementById("view__wrapper").cloneNode(true);
        if (activeView === viewId) {
            viewWrapper.classList.add("active");
        }
        viewWrapper.addEventListener("click", e => {
            changeActiveView(viewId);
            viewWrapper.classList.add("active");
        });
        
        const viewIdEl = viewWrapper.querySelector("#view-id");
        const viewUrlEl = viewWrapper.querySelector("#view-url");

        viewIdEl.textContent = viewId;
        viewUrlEl.textContent = views[viewId];

        viewsContainer.append(viewWrapper);
    }
}

function findAll(children) {
    const meshes = [];
    function _find(children) {
        for (let i = 0; i < children.length; i++) {
            let child = children[i];
            if (child.isMesh) {
                meshes.push(child);

            } else {
                child = _find(child.children);
                if (child) {
                    meshes.push(child);
                }
            }
        }
    }
    _find(children);
    
    const object = new THREE.Object3D();
    for (let i = 0; i < meshes.length; i++) {
        object.add(meshes[i]);
    }
    return object;
  }


function deleteOneCell(e) {
    if (btnState.active) {
      let cell;
      cell = raycast(mouse.mousePosition);
      if (cell) {
        engine.view.remove(cell.object);
        delete newBlockProps[cell.object.userData.id];
      } else {
        return null;
      }
    }
}
  
function saveCells() {
    const _newBlockProps = [];
    for (const id in newBlockProps) {
        _newBlockProps.push(newBlockProps[id]);
    }
    return _newBlockProps;
}

const changeStateBtn = function(e) {
    btnState.active = !btnState.active;
    if (btnState.active) {
        delCellBtn.classList.add("active-button");
        viewMeshes[activeView].material.depthTest = false;
    } else {
        delCellBtn.classList.remove("active-button");
        viewMeshes[activeView].material.depthTest = true;
    }

};

VG.EventDispatcher.bind("mouse.click", {}, deleteOneCell);

submitBtn.addEventListener("click", e => {
    e.preventDefault();
    const newCells = saveCells();

    AvvY.blockProps["properties"]["children"] = newCells;
    inputBlockProps.value = JSON.stringify(AvvY.blockProps);
    approveForm.submit();
});

saveNewViewBtn.addEventListener("click", e => {
    e.preventDefault();
    
    $('#add-view-modal').modal('hide');

    let formData = new FormData();
    formData.append("id", inputNewBlockId.value);
    formData.append('asset', inputBlockAsset.files[0]);
    const _xsrf = document.querySelector("input[name='_xsrf']").value;
    formData.append("_xsrf",_xsrf);

    fetch(`/user${AvvY.deleteLink}/view/${inputNewBlockId.value}`, {
        method: 'POST',
        body: formData,
      })
      .then((res) => {
          return res.json();
      })
      .then(response => {
        newViewForm.reset();
        const data = response.data;
        renderViewsId(data["manifest"]["meta"]["views"]);

        AvvY.blockProps["meta"]["views"] = data["manifest"]["meta"]["views"];
          
      })
      .catch(() => {
        nextView.style.display = "none";
        errorSection.classList.remove("d-none");
      })
});

delCellBtn.addEventListener("click", changeStateBtn);
